import janda
import json
import requests
import os
import re
import time
from .utils.misc import (
    choose,
    split_name,
    get_size,
    convert_html_to_pdf,
    project,
    get_size_folder,
    nums,
)
from inputimeout import inputimeout, TimeoutOccurred

simply = janda.SimplyHentai()


async def get_sim(id: str = choose().simply):
    initial = time.time()
    data = await simply.get(id)
    parser = json.loads(data)

    title = parser["title"]
    title = re.sub(r"[^\w\s]", "", title)
    print(f"Title: {title}")

    img = parser["image"]
    print(f"Total image: {len(img)}")

    tags = parser["tags"]
    tags = [tag for tag in tags]
    print(f"Tags: {tags}")

    neat_dir = f"{split_name(__file__)}-{title}"
    neat_dir = re.sub("[^A-Za-z0-9-]+", " ", neat_dir)

    neat_dir = re.sub(r"\s+", "_", neat_dir)

    set_name = parser["id"]
    set_name = set_name.split("/")[-1]

    if not os.path.exists(neat_dir):
        os.makedirs(neat_dir)

    if len(os.listdir(neat_dir)) - 1 == len(img):
        print(
            "All images already downloaded! If you're doubt kindly remove this folder and re-download"
        )
        print(f"Directory: {os.path.abspath(neat_dir)}")
        return

    for i in range(len(img)):
        start = time.time()
        r = requests.get(img[i])
        with open(f"{neat_dir}/{i+1}.jpg", "wb") as f:
            f.write(r.content)

            if os.path.exists(f"{neat_dir}/{i+1}.jpg"):
                file = get_size(f"{neat_dir}/{i+1}.jpg")
                print(
                    f"Successfully downloaded {i+1} | {file} MB | took {time.time() - start:.2f} seconds"
                )

            if len(img) == len(os.listdir(neat_dir)):

                print(
                    f"Successfully downloaded all images taken {(time.time() - initial) / 60:.2f} minutes with total size {get_size_folder(neat_dir)} MB"
                )
                print(f"Directory: {os.path.abspath(neat_dir)}")

                with open(neat_dir + "/tomoe.html", "x", encoding="utf-8") as f:
                    f.write("<html><center><body>")
                    f.write(f"<h1>{neat_dir}</h1>")

                    for i in nums(1, len(img)):

                        f.write(f'<img src="{neat_dir}/{i}.jpg"><p></p>')
                    f.write(f"{project()}")
                    f.write("</body></center></html>")
                    f.close()

                try:
                    desired = inputimeout(
                        prompt="Do you want to render it all to .pdf? (y/n) ",
                        timeout=10,
                    )
                    to_pdf = desired

                    if to_pdf == "y":
                        try:
                            source = open(f"{neat_dir}/tomoe.html")
                            output = f"{neat_dir}/{set_name}.pdf"
                            filepdf = output.rsplit("/", 1)[-1]

                            convert_html_to_pdf(source, output)
                            print(
                                f"Successfully rendered to {filepdf} | {get_size(output)} MB"
                            )

                        except Exception as e:
                            print(f"Something went wrong while converting to pdf: {e}")

                    elif to_pdf == "n":
                        print("Okay")
                        os.remove(neat_dir + "/tomoe.html")
                        return

                    else:
                        print("Invalid input")
                        os.remove(neat_dir + "/tomoe.html")
                        return

                except TimeoutOccurred:
                    print("Timeout occurred")
                    os.remove(neat_dir + "/tomoe.html")
                    exit()



import asyncio
from tomoe.pururin import get_pur
from tomoe.nhentai import get_nh
from tomoe.simplyh import get_sim
from tomoe.hentaifox import get_hfox
from tomoe.hentai2read import get_h2r
from tomoe.qhentai import get_qh
from tomoe.asmhentai import get_asm
from tomoe.utils.misc import choose, need_args


class Tomoe:
    def __init__(
        self,
        Pururin: str = choose().pururin,
        Nhentai: str = choose().nhentai,
        Simplyhentai: str = choose().simply,
        Haentaifox: str = choose().hentaifox,
        Hentai2read: str = choose().hentai2read,
        Qhentai: str = choose().qhentai,
        Asmhentai: str = choose().asmhentai,
    ):

        self.pururin = Pururin
        self.nhentai = Nhentai
        self.simply = Simplyhentai
        self.hentaifox = Haentaifox
        self.hentai2read = Hentai2read
        self.qhentai = Qhentai
        self.asmhentai = Asmhentai


Api = Tomoe()


def main():
    async def main_pururin():
        await asyncio.gather(get_pur(Api.pururin))

    async def main_nhentai():
        await asyncio.gather(get_nh(Api.nhentai))

    async def main_simply():
        await asyncio.gather(get_sim(Api.simply))

    async def main_hentaifox():
        await asyncio.gather(get_hfox(Api.hentaifox))

    async def main_hentai2read():
        await asyncio.gather(get_h2r(Api.hentai2read))

    async def main_qhentai():
        await asyncio.gather(get_qh(Api.qhentai))

    async def main_asmhentai():
        await asyncio.gather(get_asm(Api.asmhentai))

    if Api.pururin is not None:
        asyncio.run(main_pururin())

    elif Api.nhentai is not None:
        asyncio.run(main_nhentai())

    elif Api.simply is not None:
        asyncio.run(main_simply())

    elif Api.hentaifox is not None:
        asyncio.run(main_hentaifox())

    elif Api.hentai2read is not None:
        asyncio.run(main_hentai2read())

    elif Api.qhentai is not None:
        asyncio.run(main_qhentai())

    elif Api.asmhentai is not None:
        asyncio.run(main_asmhentai())

    else:
        need_args()


if __name__ == "__main__":
    main()



import argparse
import os
from xhtml2pdf import pisa


def choose():
    my_parser = argparse.ArgumentParser()

    my_parser.add_argument(
        "--nhentai", "-nhentai", action="store", metavar=("ID"), type=int
    )
    my_parser.add_argument(
        "--pururin", "-pururin", action="store", metavar=("ID"), type=int
    )
    my_parser.add_argument(
        "--simply", "-simply", action="store", metavar=("PATH_AFTER_WWW"), type=str
    )
    my_parser.add_argument(
        "--hentaifox", "-hentaifox", action="store", metavar=("ID"), type=int
    )
    my_parser.add_argument(
        "--hentai2read",
        "-hentai2read",
        nargs=2,
        action="store",
        metavar=("PATH", "CHAPTER"),
        type=str,
    )
    my_parser.add_argument(
        "--qhentai", "-qhentai", action="store", metavar=("PATH_AFTER_WWW"), type=str
    )
    my_parser.add_argument(
        "--asmhentai", "-asmhentai", action="store", metavar=("ID"), type=int
    )

    args = my_parser.parse_args()
    return args


def split_name(string):
    """Split string into name and extension

    Parameters
    ----------
    string : str
        String to split

    Returns
    -------
    name : str
        Name of the file
    """
    get_name = os.path.basename(string).split(".")[0]
    return get_name


def get_size(string):
    """Get size of a file

    Parameters
    ----------
    string : str
        Path to file

    Returns
    -------
    size : int
        Size of the file
    """
    file = round(os.path.getsize(string) / 1024 / 1024, 2)
    return file


def get_size_folder(folder):
    """Get size of a folder

    Parameters
    ----------
    folder : str
        Path to folder

    Returns
    -------
    size : int
        Size of the folder
    """
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(folder):
        for f in filenames:
            fp = os.path.join(dirpath, f)
            total_size += os.path.getsize(fp)
    return round(total_size / 1024 / 1024, 2)


def project():
    """watermark hehe"""
    return '<p><b><h1><a href="https://pypi.org/project/tomoe/">Generated from tomoe: https://pypi.org/project/tomoe</a></b><h1>'


def convert_html_to_pdf(source_html, output_filename):
    """Convert html to pdf

    Parameters
    ----------
    source_html : str
        Path to html file
    output_filename : str
        Path to pdf file
    """

    result_file = open(output_filename, "w+b")
    pisa_status = pisa.CreatePDF(source_html, dest=result_file)

    result_file.close()
    return pisa_status.err


def nums(first_number, last_number, step=1):
    """Generate access index of a list

    Parameters
    ----------
    first_number : int
        First number
    last_number : int
        Last number
    step : int
        Step

    Returns
    -------
    list
        List of numbers
    """
    return range(first_number, last_number + 1, step)


def need_args():
    """Check if arguments are given

    Returns
    -------
    bool
    """
    return print("No arguments was given")
